.code
.macro Mul32_PPU temp, highbits, xpos
  pha
  lda highbits>>2
  sta temp
  pla
  asl ; *2
  asl ; *4
  asl ; *8
  asl ; *16, finally needing top bits
  rol temp
  asl
  rol temp
  add xpos
  pha
  lda temp
  sta PPUADDR
  pla
  sta PPUADDR
.endmacro

.proc ClearOAM
  lda #-16
  ldx #0
: sta OAM_YPOS,x
  inx
  inx
  inx
  inx
  bne :-
  rts
.endproc

.proc MainLoop
  jsr wait_vblank
  jsr ClearOAM
  jsr ReadJoy

  lda CurSFXRow
  add #4
  asl
  asl
  asl
  sub #1
  sta OAM_YPOS+(4*0)
  sta OAM_YPOS+(4*1)
  sta OAM_YPOS+(4*2)
  sta OAM_YPOS+(4*3)
  ldx CurSFXColumn
  lda ColumnXPos,x
  sub #8
  sta OAM_XPOS+(4*0)
  add #8
  sta OAM_XPOS+(4*1)
  add #8
  sta OAM_XPOS+(4*2)
  add #8
  sta OAM_XPOS+(4*3)
  lda #$32
  sta OAM_TILE+(4*0)
  sta OAM_TILE+(4*1)
  sta OAM_TILE+(4*2)
  sta OAM_TILE+(4*3)
  lda #OAM_PRIORITY
  sta OAM_ATTR+(4*0)
  sta OAM_ATTR+(4*1)
  sta OAM_ATTR+(4*2)
  sta OAM_ATTR+(4*3)

  lda keydown
  and #KEY_A
  jne PressedANoMove
  lda keynew
  and #KEY_LEFT
  beq :+
    dec CurSFXColumn
: lda keynew
  and #KEY_RIGHT
  beq :+
    inc CurSFXColumn
: lda CurSFXColumn ; fix
  and #3
  sta CurSFXColumn

  lda keynew
  and #KEY_UP
  beq :+
    dec CurSFXRow
    bpl :+
      lda #24
      sta CurSFXRow
: lda keynew
  and #KEY_DOWN
  beq :+
    inc CurSFXRow
    lda CurSFXRow
    cmp #25
    bcc :+
      lda #0
      sta CurSFXRow
  :
PressedANoMove:
  lda keydown
  and #KEY_A
  jeq NoPressA

  ldx #0

  lda keynew
  and #KEY_LEFT
  beq :+
    ldx #-16
: lda keynew
  and #KEY_RIGHT
  beq :+
    ldx #16
: lda keynew
  and #KEY_UP
  beq :+
    ldx #1
: lda keynew
  and #KEY_DOWN
  beq :+
    ldx #-1
: stx 1

  lda CurSFXColumn
  pha
  lsr
  tay
  lda SFXColDataStart,y
  sta 0
  pla
  lda CurSFXColumn
  and #1
  ora 0
  sta 0
  lda CurSFXRow
  asl
  ora 0
  tay
  lda SoundData1,y
  add 1
  sta SoundData1,y

  lda 1
  beq NoPressA
  jsr wait_vblank
  lda CurSFXRow
  jsr UpdateRow
  lda #0
  sta PPUSCROLL
  sta PPUSCROLL
NoPressA:

  lda keynew
  and #KEY_START
  beq :+
  lda #0
  jsr start_sound
  lda #1
  jsr start_sound
:

  jsr update_sound
  jsr UpdateWaveform

  lda keynew
  and #KEY_B
  jne DoMenu

  jmp MainLoop

ColumnXPos:
  .byt 2*8, 5*8, 10*8, 13*8
SFXColDataStart:
  .byt 0, $80
.endproc

.proc UpdateWaveform
  lda CurSFXColumn
  lsr
  tax
  lda SFXChannels,x
  add #$25
  sta 0

  ldy #8
  ldx #0
: lda #2*8-1
  sta OAM_YPOS+(4*10),x

  txa
  asl
  sta 1
  lda retraces
  and #7
  add 1
  add #17*8
  sta OAM_XPOS+(4*10),x
  lda 0
  sta OAM_TILE+(4*10),x
  lda #0
  sta OAM_ATTR+(4*10),x
  inx
  inx
  inx
  inx
  cpx #4*10
  bne :-
  rts
.endproc

.proc DoMenu
NumOptions = 11

Loop:
  jsr wait_vblank
  jsr ReadJoy

  lda CurMenuOption
  add #4
  asl
  asl
  asl
  sub #1
  sta OAM_YPOS+(4*4)

  lda #16*8
  sta OAM_XPOS+(4*4)
  lda #'>'
  sta OAM_TILE+(4*4)
  lda #0
  sta OAM_ATTR+(4*4)

  lda keynew
  and #KEY_START
  beq :+
    lda #0
    jsr start_sound
    lda #1
    jsr start_sound
  :

  lda keynew
  and #KEY_UP
  beq :+
    dec CurMenuOption
    bpl :+
      lda #NumOptions-1
      sta CurMenuOption
  :

  lda keynew
  and #KEY_DOWN
  beq :+
    inc CurMenuOption
    lda CurMenuOption
    cmp #NumOptions
    bne :+
      lda #0
      sta CurMenuOption
  :

  lda keynew
  and #KEY_B
  jne MainLoop

  lda keynew
  and #KEY_A
  beq :+
    jsr DoMenuAction
  :

  jsr update_sound
  jsr UpdateWaveform

  jmp Loop

DoMenuAction:
  lda CurMenuOption
  asl
  tax
  lda MenuActions+1,x
  pha
  lda MenuActions+0,x
  pha
  rts
.endproc

MenuActions:
  .raddr MenuPlay
  .raddr MenuPlay1
  .raddr MenuPlay2
  .raddr MenuChannel
  .raddr MenuSwap
  .raddr MenuCopy
  .raddr MenuClear
  .raddr MenuClone
  .raddr MenuZero
  .raddr MenuDelete
  .raddr MenuExit

.proc MenuPlay
  lda #0
  jsr start_sound
  lda #1
  jsr start_sound
  rts
.endproc
.proc MenuPlay1
  lda #0
  jsr start_sound
  rts
.endproc
.proc MenuPlay2
  lda #1
  jsr start_sound
  rts
.endproc
.proc MenuChannel
  lda CurSFXColumn
  lsr
  tax
  inc SFXChannels,x

  lda SFXChannels,x
  cmp #3
  bne :+
  lda #0
  sta SFXChannels,x
:
  tay
  lda Values,y
  pha

  lda CurSFXColumn
  and #2
  asl
  tax
  pla
  sta Sound1Chan,x
  rts

Values:
  .byt 0, 8, 12
.endproc
.proc MenuSwap
  ldx #0
: lda SoundData1,x
  pha
  lda SoundData2,x
  sta SoundData1,x
  pla
  sta SoundData2,x
  inx
  cpx #50
  bne :-
  jmp NeedUpdateAll
.endproc
.proc MenuCopy
  lda #50
  sta 0

  ldx CurSFXColumn
  lda BaseForColumnSrc,x
  pha
  lda BaseForColumnDst,x
  tay
  pla
  tax

: lda SoundData1,x
  sta SoundData1,y
  inx
  iny
  dec 0
  bne :-

  jmp NeedUpdateAll
.endproc
.proc MenuClear
  ldx CurSFXColumn
  ldy BaseForColumnSrc,x
  ldx #0
  txa
: sta SoundData1,y
  iny
  inx
  cpx #50
  bne :-

  jmp NeedUpdateAll
.endproc
.proc MenuClone
  ldx CurSFXColumn
  lda BaseForColumnSrc,x
  sta 0

  add #46
  tax

  lda CurSFXRow
  asl
  add 0
  sta 0

: lda SoundData1,x
  sta SoundData1+2,x
  dex
  cpx 0
  bne :-

  jmp NeedUpdateAll
.endproc
.proc MenuZero
  ldx CurSFXColumn
  lda BaseForColumnSrc,x
  sta 0
  lda CurSFXRow
  asl
  add 0
  tay
  lda #0
  sta SoundData1+0,y
  sta SoundData1+1,y
  jmp NeedUpdateAll
.endproc
.proc MenuDelete
  ldx CurSFXColumn
  lda BaseForColumnSrc,x
  sta 0
  add #48
  sta 1

  lda CurSFXRow
  asl
  add 0
  tax
: lda SoundData1+2,x
  sta SoundData1,x
  inx
  cpx 1
  bne :-
  jmp NeedUpdateAll
.endproc
.proc MenuExit
  jmp (Vectors+2)
.endproc

BaseForColumnSrc:
  .byt 0, 0, 128, 128
BaseForColumnDst:
  .byt 128, 128, 0, 0

.proc NeedUpdateAll
  jsr wait_vblank
  lda #0
  sta PPUMASK

  jsr UpdateAllRows

  jsr wait_vblank
  lda #BG_ON|OBJ_ON
  sta PPUMASK
  rts
.endproc

.proc UpdateAllRows
  lda #0
  sta 10
: lda 10
  jsr UpdateRow
  inc 10
  lda 10
  cmp #25
  bne :-

  lda #0
  sta PPUSCROLL
  sta PPUSCROLL
  rts
.endproc

.proc UpdateRow ; A = row to update
  pha
  add #4
  Mul32_PPU 0, #$20, #2
  pla
  asl
  tax
  lda SoundData1,x
  jsr PutHex
  lda #$24
  sta PPUDATA
  lda SoundData1+1,x
  jsr PutHex
  lda #$24
  sta PPUDATA

  lda #'-'
  sta PPUDATA

  lda #$24
  sta PPUDATA

  lda SoundData2,x
  jsr PutHex
  lda #$24
  sta PPUDATA
  lda SoundData2+1,x
  jsr PutHex
  rts
.endproc

